package net.voltexstudios.applicationLobby;

import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.TextComponent;
import net.voltexstudios.applicationLobby.commands.HologramCommand;
import net.voltexstudios.applicationLobby.commands.SetSpawnCommand;
import net.voltexstudios.applicationLobby.commands.SpawnCommand;
import net.voltexstudios.applicationLobby.holograms.HologramData;
import net.voltexstudios.applicationLobby.holograms.HologramDataUtil;
import net.voltexstudios.applicationLobby.holograms.HologramService;
import net.voltexstudios.applicationLobby.listener.PlayerListener;
import net.voltexstudios.applicationLobby.util.EyeTrace;
import net.voltexstudios.applicationLobby.util.SimpleLocation;
import net.voltexstudios.applicationLobby.util.SpawnUtil;
import net.voltexstudios.applicationLobby.util.item.InteractListener;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

public final class ApplicationLobby extends JavaPlugin {

    public static final TextComponent PREFIX = Component.text("§8[§6ApplicationLobby§8] §7");

    private static ApplicationLobby instance;
    private final Map<Player, EyeTrace> eyeTraces = new HashMap<>();
    private Location spawnLocation;
    private HologramData hologramData;

    @Override
    public void onEnable() {
        // Plugin startup logic
        instance = this;

        registerListeners();
        registerCommands();

        SimpleLocation simpleLocation = SpawnUtil.getSpawnLocation();
        if(simpleLocation != null) {
            spawnLocation = simpleLocation.toLocation();
        } else {
            spawnLocation = Bukkit.getWorlds().getFirst().getSpawnLocation();
        }

        hologramData = HologramDataUtil.loadHologramData();
        if(hologramData == null) {
            hologramData = new HologramData(new HashSet<>());
            HologramDataUtil.saveHologramData();
        }
        new HologramService(hologramData.getHolograms());

    }

    @Override
    public void onDisable() {
        // Plugin shutdown logic
    }

    private void registerListeners() {
        Bukkit.getPluginManager().registerEvents(new PlayerListener(), this);
        Bukkit.getPluginManager().registerEvents(new InteractListener(), this);
    }

    private void registerCommands() {

        HologramCommand hologramCommand = new HologramCommand();
        getCommand("hologram").setExecutor(hologramCommand);
        getCommand("hologram").setTabCompleter(hologramCommand);

        getCommand("setSpawn").setExecutor(new SetSpawnCommand());
        getCommand("spawn").setExecutor(new SpawnCommand());
    }

    public static ApplicationLobby getInstance() {
        return instance;
    }

    public Location getSpawnLocation() {
        return spawnLocation;
    }

    public Map<Player, EyeTrace> getEyeTraces() {
        return eyeTraces;
    }
}
