package net.voltexstudios.applicationLobby.commands;

import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;

import net.voltexstudios.applicationLobby.ApplicationLobby;
import net.voltexstudios.applicationLobby.holograms.Hologram;
import net.voltexstudios.applicationLobby.holograms.HologramDataUtil;
import net.voltexstudios.applicationLobby.holograms.HologramService;
import net.voltexstudios.applicationLobby.holograms.HologramType;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Interaction;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Arrays;
import java.util.List;
import java.util.StringJoiner;

public class HologramCommand implements CommandExecutor, TabCompleter {

    private static final int INTERACTION_ENTITY_RANGE = 5;

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command cmd, @NotNull String label, @NotNull String[] args) {

        if (!sender.hasPermission("applicationlobby.hologram")) {
            sender.sendMessage(
                    ApplicationLobby.PREFIX
                            .append(Component.text("You do not have permission to use this command!").color(NamedTextColor.RED))
            );
            return false;
        }

        if (args.length > 3) {
            if (args[0].equalsIgnoreCase("create")) {

                if (!(sender instanceof Player player)) {
                    sender.sendMessage(
                            ApplicationLobby.PREFIX
                                    .append(Component.text("Only players can use this command!").color(NamedTextColor.RED))
                    );
                    return false;
                }

                Entity entity = player.getTargetEntity(INTERACTION_ENTITY_RANGE);
                if (entity == null || !entity.getType().equals(EntityType.INTERACTION)) {
                    player.sendMessage(
                            ApplicationLobby.PREFIX
                                    .append(Component.text("You must be looking at an interaction entity to use this command!").color(NamedTextColor.RED))
                    );
                    return false;
                }

                Interaction interaction = (Interaction) entity;

                HologramType type = HologramType.valueOf(args[1]);
                String name = args[2];
                StringJoiner dataJoiner = new StringJoiner(" ");
                for (int i = 3; i < args.length; i++) {
                    dataJoiner.add(args[i]);
                }
                Hologram hologram = HologramService.getInstance().createHologram(type, name, interaction.getUniqueId(), dataJoiner.toString());
                if (hologram == null) {
                    sender.sendMessage(ApplicationLobby.PREFIX.append(Component.text("Hologram creation failed!").color(NamedTextColor.RED)));
                    return false;
                }

                HologramDataUtil.saveHologramData();

                sender.sendMessage(ApplicationLobby.PREFIX.append(Component.text("Hologram created!").color(NamedTextColor.GREEN)));
                return true;
            }
        }

        if (args.length == 1) {
            if (args[0].equalsIgnoreCase("list")) {
                sender.sendMessage(ApplicationLobby.PREFIX.append(Component.text("Holograms:").color(NamedTextColor.GRAY)));
                HologramService.getInstance().getHolograms().forEach(hologram -> {
                    sender.sendMessage(ApplicationLobby.PREFIX.append(Component.text("Name: " + hologram.getName() + " Type: " + hologram.getType().name()).color(NamedTextColor.GRAY)));
                });
                return true;
            }
        }

        return false;
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender commandSender, @NotNull Command command, @NotNull String s, @NotNull String[] strings) {
        if (strings.length == 1) {
            return List.of("create", "delete", "list");
        }
        if (strings.length == 2) {
            if (strings[0].equalsIgnoreCase("create")) {
                return Arrays.stream(HologramType.values()).toList().stream().map(Enum::name).toList();
            }
            else if (strings[0].equalsIgnoreCase("delete")) {
                List<String> hologramNames = new java.util.ArrayList<>(HologramService.getInstance().getHolograms().stream().map(Hologram::getName).toList());
                hologramNames.add("all");
                return hologramNames;
            }
        }
        return null;
    }
}
