package net.voltexstudios.applicationLobby.holograms;

import net.voltexstudios.applicationLobby.holograms.actions.Clickable;
import net.voltexstudios.applicationLobby.holograms.actions.Hoverable;
import org.bukkit.Bukkit;
import org.bukkit.entity.Display;
import org.bukkit.entity.Interaction;
import org.bukkit.entity.Player;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public abstract class Hologram implements Hoverable, Clickable {

    protected final Map<UUID, UUID> displayMap = new HashMap<>();
    private final String name;
    private final UUID interactionUUID;
    private final HologramType type;

    public Hologram(String name, HologramType type, UUID interactionUUID) {
        this.name = name;
        this.type = type;
        this.interactionUUID = interactionUUID;
    }

    public abstract void addDisplay(UUID PlayerUUID);

    protected Player getPlayer(UUID playerUUID) {
        return Bukkit.getServer().getPlayer(playerUUID);
    }

    protected Interaction getInteraction(Player player) {
        return (Interaction) player.getWorld().getEntity(interactionUUID);
    }

    protected Display getDisplay(Player player) {
        return (Display) player.getWorld().getEntity(displayMap.get(player.getUniqueId()));
    }

    public void removeDisplay(UUID playerUUID) {
        Display display = (Display) Bukkit.getServer().getEntity(displayMap.get(playerUUID));
        if (display == null) {
            return;
        }

        display.remove();
        displayMap.remove(playerUUID);
    }

    public UUID getInteractionUUID() {
        return interactionUUID;
    }

    public String getName() {
        return name;
    }

    public HologramType getType() {
        return type;
    }
}
