package net.voltexstudios.applicationLobby.holograms.types;

import net.kyori.adventure.text.Component;
import net.voltexstudios.applicationLobby.ApplicationLobby;
import net.voltexstudios.applicationLobby.holograms.Hologram;
import net.voltexstudios.applicationLobby.holograms.HologramType;
import org.bukkit.Color;
import org.bukkit.entity.*;
import org.bukkit.util.Transformation;
import org.bukkit.util.Vector;

import java.util.UUID;

public class TextHologram extends Hologram {

    private static final float HOVER_SCALING = 1.3f;

    private final String text;

    public TextHologram(String name, UUID interactionUUID, String text) {
        super(name, HologramType.TEXT, interactionUUID);
        this.text = text;
    }

    @Override
    public void addDisplay(UUID playerUUID) {
        Player player = getPlayer(playerUUID);
        if (player == null) {
            return;
        }

        Interaction interaction = getInteraction(player);
        if (interaction == null) {
            return;
        }

        TextDisplay display = createDisplay(player, new Vector(-0.05, -1, 0));
        player.showEntity(ApplicationLobby.getInstance(), display);

        displayMap.put(playerUUID, display.getUniqueId());
    }

    private TextDisplay createDisplay(Player player, Vector offset) {
        TextDisplay display = (TextDisplay) player.getWorld().spawnEntity(getInteraction(player).getLocation().clone()
                .setDirection(new Vector(1, 0, 0))
                .add(offset),
                EntityType.TEXT_DISPLAY);
        display.setAlignment(TextDisplay.TextAlignment.CENTER);
        display.setBillboard(TextDisplay.Billboard.FIXED);
        display.setDefaultBackground(false);
        display.setBackgroundColor(Color.fromARGB(0, 0, 0, 0));
        display.text(Component.text(text));
        display.setVisibleByDefault(false);

        Transformation transformation = display.getTransformation();
        transformation.getScale().mul(8.313f, 8.313f, 8.313f);
        display.setTransformation(transformation);

        return display;
    }

    @Override
    public void onClick(Player player) {
        //Do nothing for now
        player.sendMessage("Klicked " + getName());
    }

    @Override
    public void onHover(Player player) {
        player.playSound(player, "custom:hover_sound", 0.5f, 1);

        Display display = getDisplay(player);
        if (display == null) {
            player.sendMessage(
                    "§cAn error occurred while trying to hover over the NPC. Please try again later."
            );
            return;
        }

        Transformation transformation = display.getTransformation();
        transformation.getScale().mul(HOVER_SCALING, HOVER_SCALING, HOVER_SCALING);
        display.setInterpolationDelay(0);
        display.setInterpolationDuration(3); // smooth the scaling
        display.setTransformation(transformation);
    }

    @Override
    public void onUnhover(Player player) {
        Display display = getDisplay(player);
        if (display == null) {
            player.sendMessage(
                    "§cAn error occurred while trying to hover over the NPC. Please try again later."
            );
            return;
        }

        Transformation transformation = display.getTransformation();
        transformation.getScale().div(HOVER_SCALING, HOVER_SCALING, HOVER_SCALING);
        display.setInterpolationDelay(0);
        display.setInterpolationDuration(3); // smooth the scaling
        display.setTransformation(transformation);
    }
}
