package net.voltexstudios.applicationLobby.listener;

import net.voltexstudios.applicationLobby.holograms.Hologram;
import net.voltexstudios.applicationLobby.holograms.HologramService;
import net.voltexstudios.applicationLobby.util.PlayerUtil;
import org.bukkit.entity.Display;
import org.bukkit.entity.Interaction;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntitySpawnEvent;
import org.bukkit.event.entity.FoodLevelChangeEvent;
import org.bukkit.event.player.*;
import org.bukkit.util.RayTraceResult;

public class PlayerListener implements Listener {

    @EventHandler
    public void onFoodLevelChange(FoodLevelChangeEvent event) {
        event.setCancelled(true);
    }

    @EventHandler
    public void onDamage(EntityDamageEvent event) {
        event.setCancelled(true);
    }

    @EventHandler
    public void onEntitySpawn(EntitySpawnEvent event) {
        if (!(event.getEntity() instanceof Display)) {
            event.setCancelled(true);
        }
    }

    @EventHandler
    public void onJoin(PlayerJoinEvent event) {
        PlayerUtil.handlePlayerJoin(event.getPlayer());
    }

    @EventHandler
    public void onLeave(PlayerQuitEvent event) {
        PlayerUtil.handlePlayerQuit(event.getPlayer());
    }

    @EventHandler
    public void onTexturepackLoad(PlayerResourcePackStatusEvent event) {
        if (event.getStatus().equals(PlayerResourcePackStatusEvent.Status.SUCCESSFULLY_LOADED)) {
            HologramService.getInstance().addPlayerDisplays(event.getPlayer());
        }
    }

    @EventHandler
    public void onEntityInteract(PlayerInteractEntityEvent event) {
        if (event.getRightClicked() instanceof Interaction interaction) {
            event.setCancelled(true);

            Hologram hologram = HologramService.getInstance().getHologram(interaction.getUniqueId());
            if (hologram == null) {
                return;
            }

            HologramService.getInstance().handleHologramClick(event.getPlayer(), hologram);
        }
    }

    @EventHandler
    public void onInteraction(PlayerInteractEvent event) {
        Player player = event.getPlayer();
        if (event.getAction() != Action.RIGHT_CLICK_AIR) {
            return;
        }

        RayTraceResult result = player.getWorld().rayTraceEntities(
                player.getEyeLocation(),
                player.getEyeLocation().getDirection(),
                15,
                entity -> entity instanceof Interaction
        );

        if (result == null) {
            return;
        }

        Interaction interaction = (Interaction) result.getHitEntity();
        if (interaction == null) {
            return;
        }

        Hologram hologram = HologramService.getInstance().getHologram(interaction.getUniqueId());
        if (hologram == null) {
            return;
        }

        HologramService.getInstance().handleHologramClick(player, hologram);
    }


}
