package net.voltexstudios.applicationLobby.phone;

import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.minimessage.MiniMessage;
import net.voltexstudios.applicationLobby.phone.items.CommandAppItem;
import net.voltexstudios.applicationLobby.phone.items.EmptyAppItem;
import net.voltexstudios.applicationLobby.phone.items.LinkAppItem;
import net.voltexstudios.applicationLobby.util.CharRepo;
import net.voltexstudios.applicationLobby.util.item.InteractableItem;
import net.voltexstudios.applicationLobby.util.item.ItemRegistry;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import xyz.xenondevs.inventoryaccess.component.AdventureComponentWrapper;
import xyz.xenondevs.invui.gui.Gui;
import xyz.xenondevs.invui.item.ItemProvider;
import xyz.xenondevs.invui.item.builder.ItemBuilder;
import xyz.xenondevs.invui.window.Window;


public class PhoneManager {

    private static PhoneManager instance;
    private final PhoneItem phoneItem;

    private PhoneManager() {
        phoneItem = new PhoneItem(
                new ItemBuilder(Material.PAPER)
                        .setDisplayName(new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#7d7d7d>[</color><color:#00ebeb>Handy</color><color:#7d7d7d>]</color>")))
                        .setCustomModelData(37).get()
        );

        ItemRegistry.registerItem(phoneItem);
    }

    public InteractableItem getPhoneItem() {
        return phoneItem;
    }

    public void openGUI(Player player) {
        Window window = Window.split()
                .setTitle(CharRepo.getNeg(48) + "§f" + CharRepo.PHONE_GUI)
                .setUpperGui(getUpperGui())
                .setLowerGui(getLowerGui())
                .setViewer(player)
                .build();

        window.open();
    }

    private Gui getUpperGui() {

        AppItem applicationPlotItem = new CommandAppItem(
                "APPLICATION PLOT COMMAND",
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#66b525>ᴘʟᴏᴛ</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#c98600>Phase I</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#8f8f8f>Bewirb Dich , Lerne Tips und Tricks,</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#8f8f8f>Verbessere Dich</color>")),
                new AdventureComponentWrapper(Component.empty()),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#8f8f8f>Klicke um zu Starten</color>"))
        );

        AppItem trialPlotItem = new CommandAppItem(
                "TRIAL PLOT COMMAND",
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#fcb92a>ᴛʀɪᴀʟ</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#c98600>Phase II</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>Suche dir eines von 3 Themen aus und</gray>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>setze dieses in 7 Tagen um!</gray>")),
                new AdventureComponentWrapper(Component.empty()),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>Klicke zum Starten</gray>"))
        );

        AppItem tutorialItem = new EmptyAppItem(
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#9359e9>ᴛᴜᴛᴏʀɪᴀʟ</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>Hier lernst du alle wichtigen Schritte </gray>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>zum Bauen eines Plots kennen!</gray>")),
                new AdventureComponentWrapper(Component.empty()),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<dark_red>Coming Soon!</dark_red>"))
        );

        AppItem mailItem = new CommandAppItem(
                "MAIL COMMAND",
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#03a99f>ᴍᴀɪʟ</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>Hier erhältst du deine Bewertungen und Tipps!</gray>")),
                new AdventureComponentWrapper(Component.empty()),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>Klicke um dein Postfach zu öffnen</gray>"))
        );

        return Gui.normal()
                .setStructure(
                        "x x x x x x x x x",
                        "x o o o x y y y x",
                        "x o o o x y y y x",
                        "x x x x x x x x x",
                        "x z z z x s s s x",
                        "x z z z x s s s x"
                )
                .addIngredient('o', applicationPlotItem)
                .addIngredient('y', trialPlotItem)
                .addIngredient('z', tutorialItem)
                .addIngredient('s', mailItem)
                .build();
    }

    private Gui getLowerGui() {

        AppItem gamesItem = new EmptyAppItem(
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#d261cc>ɢᴀᴍᴇѕ</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>Überbrücke Wartezeit und spiele MiniGames</gray>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>mit deinen Freunden!</gray>")),
                new AdventureComponentWrapper(Component.empty()),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<dark_red>Coming Soon!</dark_red>"))
        );

        AppItem settingsItem = new EmptyAppItem(
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#4cb6db>ѕᴇᴛᴛɪɴɢѕ</color>")),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<gray>Hier kannst du deine Settings umstellen!</gray>")),
                new AdventureComponentWrapper(Component.empty()),
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<dark_red>Coming Soon!</dark_red>"))
        );

        AppItem twitterItem = new LinkAppItem(
                "https://x.com/Voltexstudio",
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#00bcf7>ᴛᴡɪᴛᴛᴇʀ</color>"))
        );

        AppItem instagramItem = new LinkAppItem(
                "https://www.instagram.com/voltex.studios/",
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#f73a5a>ɪɴѕᴛᴀɢʀᴀᴍ</color>"))
        );

        AppItem tiktokItem = new LinkAppItem(
                "https://www.tiktok.com/@voltexstudiosofficial",
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#b6c1cc>ᴛɪᴋᴛᴏᴋ</color>"))
        );

        AppItem youtubeItem = new LinkAppItem(
                "https://www.youtube.com/@VoltexStudio",
                new AdventureComponentWrapper(MiniMessage.miniMessage().deserialize("<color:#de3a24>ʏᴏᴜᴛᴜʙᴇ</color>"))
        );

        return Gui.normal()
                .setStructure(
                        "x f f f x g g g x",
                        "x f f f x g g g x",
                        "x x x x x x x x x",
                        "x a x b x c x d x"
                )
                .addIngredient('f', gamesItem)
                .addIngredient('g', settingsItem)
                .addIngredient('a', twitterItem)
                .addIngredient('b', instagramItem)
                .addIngredient('c', tiktokItem)
                .addIngredient('d', youtubeItem)
                .build();
    }

    public static PhoneManager getInstance() {
        if(instance == null) {
            instance = new PhoneManager();
        }
        return instance;
    }
}
